// This texture contains current scene rendered by xRenderWorld()
// without downsampling.
// Also you may use SCENECURRENT2X, SCENECURRENT4X and SCENECURRENT8X
// semantics to get downsampled scene texture (e.g. for bloom sample
// 800x600 backbuffer will be downsampled to 400x300, 200x150 and 100x75
// textures).
// And if you need to get previous frame texture you may use this semantics:
// SCENEPREVIOUS, SCENEPREVIOUS2X, SCENEPREVIOUS4X and SCENEPREVIOUS8X




float bloom_power = 1.65;
float bloom_power2 = .8;


texture sceneTexture : SCENECURRENT;

// Texture sampler to get access to texture pixels for shader
sampler sceneSample = sampler_state
{
	// Assing texture to sampler
	Texture   = <sceneTexture>;
	// We always needs clamping for textures
	AddressU  = Clamp;
	AddressV  = Clamp;
	// Linear texture filtering, also you may use "Point" or "None"
	MinFilter = Linear;
	MagFilter = Linear;
	MipFilter = Linear;
};

// This texture contains current buffer state after appling other posteffects
// with 8x downsampling.
// Also you may use CURRENTBUFFER, CURRENTBUFFER2X and CURRENTBUFFER4X
// semantics.
texture currentTexture : CURRENTBUFFER2X;

// Texture sampler to get access to texture pixels for shader
sampler currentSample = sampler_state
{
	// Assing texture to sampler
	Texture   = <currentTexture>;
	// We always needs clamping for textures
	AddressU  = Clamp;
	AddressV  = Clamp;
	// Linear texture filtering, also you may use "Point" or "None"
	MinFilter = Linear;
	MagFilter = Linear;
	MipFilter = Linear;
};

// Also, if you need to get scene depth you may use DEPTHMAP semantic, e.g.:
/*
texture depthTexture : DEPTHMAP;
sampler depthSample = sampler_state
{
	Texture   = <depthTexture>;
	AddressU  = Clamp;
	AddressV  = Clamp;
	MinFilter = Linear;
	MagFilter = Linear;
	MipFilter = Linear;
};
*/

// Constants for Gausian blur (http://en.wikipedia.org/wiki/Gaussian_blur)
static const int kernelSize = 13;
static const float downSize = 210.0f;

float2 pixelKernel[kernelSize] =
{
    { -6.0f / downSize, 0.0f },
    { -5.0f / downSize, 0.0f },
    { -4.0f / downSize, 0.0f },
    { -3.0f / downSize, 0.0f },
    { -2.0f / downSize, 0.0f },
    { -1.0f / downSize, 0.0f },
    {  0.0f,            0.0f },
    {  1.0f / downSize, 0.0f },
    {  2.0f / downSize, 0.0f },
    {  3.0f / downSize, 0.0f },
    {  4.0f / downSize, 0.0f },
    {  5.0f / downSize, 0.0f },
    {  6.0f / downSize, 0.0f },
};

float2 pixelKernelV[kernelSize] =
{
    { 0.0f, -6.0f / downSize },
    { 0.0f, -5.0f / downSize },
    { 0.0f, -4.0f / downSize },
    { 0.0f, -3.0f / downSize },
    { 0.0f, -2.0f / downSize },
    { 0.0f, -1.0f / downSize },
    { 0.0f,  0.0f            },
    { 0.0f,  1.0f / downSize },
    { 0.0f,  2.0f / downSize },
    { 0.0f,  3.0f / downSize },
    { 0.0f,  4.0f / downSize },
    { 0.0f,  5.0f / downSize },
    { 0.0f,  6.0f / downSize },
};

static const float blurWeights[kernelSize] = 
{
    0.002216,
    0.008764,
    0.026995,
    0.064759,
    0.120985,
    0.176033,
    0.199471,
    0.176033,
    0.120985,
    0.064759,
    0.026995,
    0.008764,
    0.002216,
};

float BloomWeights[9]={0,0.092,0.081,0.071,0.061,0.051,0.041,0.031,0.021};
float BloomOffset=0.004f;


// Pixel shader for bloom
float4 PSBloom(float2 texCoords : TEXCOORD0) : COLOR
{
	// We getting pixel value from scene buffer
    float4 color = tex2D(currentSample, texCoords);
	// And bloom it
		return 1.0f * (color - bloom_power2);
    //return 4.0f * (color - 0.75);
	// Actually, we can do with pixel's color anything. E.g.
	// grayscale filter:
	// return (color.x + color.y + color.z) / 3.0f;
	// Or mixing with other texture: 
	// float4 otherColor = tex2D(otherSampler, texCoords);
	// return color * otherColor;
}

// Gausian blur pixel shaders
float4 PSBlurH(float2 texCoords : TEXCOORD0) : COLOR
{
    float4 color = 0.0f;

   	for (int i = 1; i < 9; i++) color +=tex2D(currentSample,texCoords+float2(i*BloomOffset,0))*BloomWeights[i];
    for (int i = 1; i < 9; i++) color +=tex2D(currentSample,texCoords+float2(-i*BloomOffset,0))*BloomWeights[i];
    
    return color * bloom_power;
}

float4 PSBlurV(float2 texCoords : TEXCOORD0) : COLOR
{
    float4 color = 0.0f;
   		for (int i = 1; i < 9; i++) color +=tex2D(currentSample,texCoords+float2(0,i*BloomOffset))*BloomWeights[i];
     	for (int i = 1; i < 9; i++) color +=tex2D(currentSample,texCoords+float2(0,-i*BloomOffset))*BloomWeights[i];



		
		float contrast = 1.70f;
		float brightness = 0.15f;
		
		float4 cont = tex2D(sceneSample, texCoords);
	 	cont.rgb /= cont.a;
  	cont.rgb = ((cont.rgb - 0.5f) * max(contrast, 0)) + 0.5f;
  	cont.rgb += brightness;
  	cont.rgb *= cont.a;
 	


    return color * bloom_power + cont;
}




// Shaders techniques. Shader must contain at least one technique.
technique MainTechnique
{
	// Sequence of technique passes. This passes will performs on stage.
	pass Bloom
	{
		PixelShader	= compile ps_2_0 PSBloom();
	}

	pass BlurH
	{
		PixelShader	= compile ps_2_0 PSBlurH();
	}


	pass BlurV
	{
		PixelShader	= compile ps_2_0 PSBlurV();
	}


}