

const float4x4 MatWorldViewProj : MATRIX_WORLDVIEWPROJ;
float4x4 textureMatrix : MATRIX_TEXTURE0;
const float4 instances[200] : INSTANCES_ARRAY;

const texture textureDiffuse : TEXTURE_0;
sampler2D samplerDiffuse = sampler_state
{
	Texture   = <textureDiffuse>;
	ADDRESSU  = WRAP;
	ADDRESSV  = WRAP;
	ADDRESSW  = WRAP;
	MAGFILTER = LINEAR;
	MINFILTER = LINEAR;
	MIPFILTER = LINEAR;
};


struct VSInput
{
	float4 Position  : POSITION0;
	float2 TexCoords : TEXCOORD0;
};
struct VSInput5
{
	float4 Position  : POSITION0;
	float2 TexCoords : TEXCOORD0;
	
	float4 indices   : BLENDWEIGHT;
};

struct VSOutput
{
	float4 Position  : POSITION0;
	float4 Depth     : TEXCOORD1;
	float2 TexCoords : TEXCOORD0;
};

void VSDepth(in VSInput IN, out VSOutput OUT) 
{
	OUT.Position  = mul(IN.Position, MatWorldViewProj);
	OUT.Depth     = OUT.Position;
	OUT.TexCoords = IN.TexCoords;
}

void VSDepthInstanced(in VSInput5 IN, out VSOutput OUT) 
{
	float4x4 worldMatrix;
	int index      = IN.indices.x;
	worldMatrix[0] = float4(instances[index * 4 + 0].x, instances[index * 4 + 1].x, instances[index * 4 + 2].x, 0.0f);
	worldMatrix[1] = float4(instances[index * 4 + 0].y, instances[index * 4 + 1].y, instances[index * 4 + 2].y, 0.0f);
	worldMatrix[2] = float4(instances[index * 4 + 0].z, instances[index * 4 + 1].z, instances[index * 4 + 2].z, 0.0f);
	worldMatrix[3] = float4(instances[index * 4 + 0].w, instances[index * 4 + 1].w, instances[index * 4 + 2].w, 1.0f);
	OUT.Position  = mul(mul(IN.Position, worldMatrix), MatWorldViewProj);
	OUT.Depth     = OUT.Position;
	OUT.TexCoords = IN.TexCoords;	
}

float4 PSDepth(VSOutput IN): COLOR0
{
	float depth = IN.Depth.z / IN.Depth.w;
	return float4(depth, depth, depth, tex2D(samplerDiffuse, IN.TexCoords).a);
}


technique Depth
{
	pass p0 
	{
		AlphaBlendEnable = 0;
		AlphaTestEnable  = 1;
		AlphaRef         = 128;
		AlphaFunc        = 5;
		CullMode         = NONE;
		VertexShader     = compile vs_2_0 VSDepth();
		PixelShader      = compile ps_2_0 PSDepth();
	}
}


technique DepthInstances
{
	pass p0 
	{
		AlphaBlendEnable = 0;
		AlphaTestEnable  = 1;
		AlphaRef         = 128;
		AlphaFunc        = 5;
		CullMode         = NONE;
		VertexShader     = compile vs_2_0 VSDepthInstanced();
		PixelShader      = compile ps_2_0 PSDepth();
	}
}
