
//##################  Varriables  ##################
const float4x4 MatWorldViewProj : MATRIX_WORLDVIEWPROJ;

float4 skycolor;
float4 skycolor2;

float StarsAlp = 0;
float CloudsPos;
float CloudPower = 0.8;

float CloudDensity = 1.75;
float3 CloudColor = float3(1.0f,1.0f,1.0f);
   
float time				: CURRENT_TIME;
float3 LightDir   : LIGHT0_DIRECTION;
   
const texture StarsTex;
sampler StarsSample = sampler_state {
    Texture   = <StarsTex>;
    ADDRESSU  = WRAP;
    ADDRESSV  = WRAP;
    ADDRESSW  = WRAP;
    MAGFILTER = LINEAR;
    MINFILTER = LINEAR;
    MIPFILTER = LINEAR;
};

const texture StarsMaskTex;
sampler MaskSample = sampler_state {
    Texture   = <StarsMaskTex>;
    ADDRESSU  = WRAP;
    ADDRESSV  = WRAP;
    ADDRESSW  = WRAP;
    MAGFILTER = LINEAR;
    MINFILTER = LINEAR;
    MIPFILTER = LINEAR;
};

const texture CloudsTex1;
sampler Clouds1Sample = sampler_state {
    Texture   = <CloudsTex1>;
    ADDRESSU  = WRAP;
    ADDRESSV  = WRAP;
    ADDRESSW  = WRAP;
    MAGFILTER = LINEAR;
    MINFILTER = LINEAR;
    MIPFILTER = LINEAR;
};

const texture CloudsTex2;
sampler Clouds2Sample = sampler_state {
    Texture   = <CloudsTex2>;
    ADDRESSU  = WRAP;
    ADDRESSV  = WRAP;
    ADDRESSW  = WRAP;
    MAGFILTER = LINEAR;
    MINFILTER = LINEAR;
    MIPFILTER = LINEAR;
};

const texture CloudsTex3;
sampler Clouds3Sample = sampler_state {
    Texture   = <CloudsTex3>;
    ADDRESSU  = WRAP;
    ADDRESSV  = WRAP;
    ADDRESSW  = WRAP;
    MAGFILTER = LINEAR;
    MINFILTER = LINEAR;
    MIPFILTER = LINEAR;
};



//###############  Input VS  ###############
struct vis 
{
   float4 Pos				: POSITION;
   float2 texCoords : TEXCOORD0;
};

//###############  Output VS  ##############
struct pis 
{
   float4 Pos    		: POSITION;

   float2 texCoords : TEXCOORD0;
   float3 viewVec 	: TEXCOORD1;
};

void vsSky( in vis IN, out pis Out ) 
{
   Out.Pos 				= mul(IN.Pos, MatWorldViewProj);
   Out.viewVec 		= IN.Pos.xyz;   
   Out.texCoords 	= IN.texCoords;		
}


//##################  PS  ##################
//===============  Diffuse  ================
float4 psSky( in pis IN ) : COLOR 
{
	
	float Hori 			= 0.0;
	float HoriFade 	= 8.0;
	
	float4 Result = lerp(skycolor, skycolor2, smoothstep(Hori, HoriFade, IN.viewVec.y));	
	//lerp(skycolor , skycolor * .75 , saturate((IN.viewVec.y - Hori) / (HoriFade - Hori)));

	// Stars
	if ( StarsAlp > 0 ) {
		float4 Stars = tex2D( StarsSample, IN.texCoords * 4 ) ;
		Stars *= tex2D( MaskSample, ( IN.texCoords * 2 ) + ( time / 75 ) ) * StarsAlp;
		
		Hori 			= 2.0;
		HoriFade 	= 6.0;
		
		Result += lerp(0 , Stars * 5, saturate((IN.viewVec.y - Hori) / (HoriFade - Hori)));
	}
	
	return Result;
}


float4 psClouds( in pis IN ) : COLOR 
{
			
	float2 CloudUV1 = (IN.texCoords) * 2; 	// MASK
	float2 CloudUV2 = (IN.texCoords) * 4;
	
	// Samples
	float MaskCloud = tex2D( Clouds2Sample, ( CloudUV1 ) + CloudsPos) * CloudDensity;
	float Clouds = saturate( ( tex2D( Clouds1Sample, ( CloudUV2 ) + CloudsPos) - MaskCloud));
	float3 Normalmap = tex2D( Clouds3Sample, ( CloudUV2 ) + CloudsPos ) * 2 - MaskCloud; 
	
	// Light Calc
	float Light = 0.25f + saturate( ( 0.4f + ( 1 - Clouds ) * 0.6f ) * ( 0.4f + ( dot( -LightDir, Normalmap ) ) * 0.5f ) );
	
	// Fade Calc
	float Hori 			= 0.0;
	float HoriFade 	= 3.0;		
	float Fade = lerp(0, Clouds, saturate((IN.viewVec.y - Hori) / (HoriFade - Hori)));	
		
	// Clouds Power
	Light *= CloudPower;
	
	return float4( Light * CloudColor, Fade);
}


technique Sky 
{
	pass p0 
	{
		vertexshader	= compile vs_2_0 vsSky();
		pixelshader		= compile ps_2_0 psSky();
	}
}

technique Clouds 
{
	pass p0 
	{
		AlphaBlendEnable = true;
		SrcBlend			   = SrcAlpha;
		DestBlend			   = InvSrcAlpha;    
    ZWriteEnable     = false;
    
		vertexshader	= compile vs_2_0 vsSky();
		pixelshader		= compile ps_2_0 psClouds();
	}
}