// Semantic //////////////////////////////////////////////
float4x4 matWorld 					: MATRIX_WORLD;
float4x4 matWorldViewProj 	: MATRIX_WORLDVIEWPROJ;
float4x4 ViewInv						: MATRIX_VIEWINVERSE; 
float3x3 TBN = {float3(1,0,0),float3(0,0,1),float3(0,1,0)};

// Xors Data /////////////////////////////////////////////
float3 lightSourceDir : LIGHT0_DIRECTION;
float3 LightPos 			: LIGHT0_POSITION;
float4 lightColor     : LIGHT0_COLOR;
float4 ambientColor   : COLOR_AMBIENT;

float4 CamPos					: CAMERA_POSITION;
float  time						: CURRENT_TIME;


// Water Visual Vars /////////////////////////////////////
float Alpha 		= 1.0f;										// Water Alpha
float Softness 	= 0.2f;										// Soft Edges

float2 TexScale = { 1.0f, 1.0f };					// Normals Tex Scale

float3 FogColor;													// Dist Fog Color
float3 ReflectionColor;										// Reflection Color
float3 RefractionColor;										// Refraction Color

const float Refract;											// Refraction Intensity
const float Reflect;											// Reflection Intensity


// Waves Vars ////////////////////////////////////////////
float2 ScaleVec 	= { 100, 100 };												// Wave Area
float4 WaterSpeed = { -0.004f, -0.0045f, 0.1f, 0.02f };	// Wave Mov Speed
float  WaveHeight = .009;																// Wave Height
float2 WaterScale	= { 100, 100 };												// Wave Res


// Others ////////////////////////////////////////////////
float3 NormalWater;
float ResX;
float ResY;

/*
static const float downSize = 512.0f;
static const int kernelSize = 13;
float2 pixelKernelV[kernelSize] ={{ 0.0f, -6.0f / downSize },{ 0.0f, -5.0f / downSize },{ 0.0f, -4.0f / downSize },{ 0.0f, -3.0f / downSize },{ 0.0f, -2.0f / downSize },{ 0.0f, -1.0f / downSize },{ 0.0f,  0.0f            },{ 0.0f,  1.0f / downSize },{ 0.0f,  2.0f / downSize },{ 0.0f,  3.0f / downSize },{ 0.0f,  4.0f / downSize },{ 0.0f,  5.0f / downSize },{ 0.0f,  6.0f / downSize },};
float2 pixelKernel[kernelSize] ={{ -6.0f / downSize, 0.0f },{ -5.0f / downSize, 0.0f },{ -4.0f / downSize, 0.0f },{ -3.0f / downSize, 0.0f },{ -2.0f / downSize, 0.0f },{ -1.0f / downSize, 0.0f },{  0.0f,            0.0f },{  1.0f / downSize, 0.0f },{  2.0f / downSize, 0.0f },{  3.0f / downSize, 0.0f },{  4.0f / downSize, 0.0f },{  5.0f / downSize, 0.0f },{  6.0f / downSize, 0.0f },};
static const float blurWeights[kernelSize] = {0.002216,0.008764,0.026995,0.064759,0.120985,0.176033,0.199471,0.176033,0.120985,0.064759,0.026995,0.008764, 0.002216,};
*/

// Depth Texture ///////////////////////////////////////////
texture DepthTexture;
sampler DEPTH 	= sampler_state {
		Texture   	= <DepthTexture>;
    AddressU 		= Clamp;
    AddressV 		= Clamp;
    MinFilter 	= Linear;
    MagFilter 	= Linear;
    MipFilter 	= Linear;
};

// Reflection & Reflaction /////////////////////////////////
const texture tReflection;
sampler TexReflection=sampler_state {
    Texture   = <tReflection>;
    ADDRESSU  = CLAMP;
    ADDRESSV  = CLAMP;
    ADDRESSW  = CLAMP;
    MAGFILTER = LINEAR;
    MINFILTER = LINEAR;
    MIPFILTER = LINEAR;
};

const texture tScreen;
sampler TexScreen=sampler_state {
    Texture   = <tScreen>;
    ADDRESSU  = CLAMP;
    ADDRESSV  = CLAMP;
    ADDRESSW  = CLAMP;
    MAGFILTER = LINEAR;
    MINFILTER = LINEAR;
    MIPFILTER = LINEAR;
};

// Normal Textures /////////////////////////////////////////
const texture tNormalW;
sampler TexNormalW=sampler_state {
    Texture   = <tNormalW>;
    ADDRESSU  = WRAP;
    ADDRESSV  = WRAP;
    ADDRESSW  = WRAP;
    MAGFILTER = LINEAR;
    MINFILTER = LINEAR;
    MIPFILTER = LINEAR;
};
const texture tNormalN;
sampler TexNormalN=sampler_state {
    Texture   = <tNormalN>;
    ADDRESSU  = WRAP;
    ADDRESSV  = WRAP;
    ADDRESSW  = WRAP;
    MAGFILTER = LINEAR;
    MINFILTER = LINEAR;
    MIPFILTER = LINEAR;
};

const texture tNormalH1;
sampler TexWaves=sampler_state {
    Texture   = <tNormalH1>;
    ADDRESSU  = WRAP;
    ADDRESSV  = WRAP;
    ADDRESSW  = WRAP;
    MAGFILTER = LINEAR;
    MINFILTER = LINEAR;
    MIPFILTER = LINEAR;
};

const texture tNormalH2;
sampler TexWaves2=sampler_state {
    Texture   = <tNormalH2>;
    ADDRESSU  = WRAP;
    ADDRESSV  = WRAP;
    ADDRESSW  = WRAP;
    MAGFILTER = LINEAR;
    MINFILTER = LINEAR;
    MIPFILTER = LINEAR;
};


const texture FoamTexture;
sampler FoamT=sampler_state {
    Texture   = <FoamTexture>;
    ADDRESSU  = WRAP;
    ADDRESSV  = WRAP;
    ADDRESSW  = WRAP;
    MAGFILTER = LINEAR;
    MINFILTER = LINEAR;
    MIPFILTER = POINT;
};

const texture FoamTexture2;
sampler FoamT2=sampler_state {
    Texture   = <FoamTexture2>;
    ADDRESSU  = WRAP;
    ADDRESSV  = WRAP;
    ADDRESSW  = WRAP;
    MAGFILTER = LINEAR;
    MINFILTER = LINEAR;
    MIPFILTER = POINT;
};


struct VS_INPUT
{
	float4 Pos 				: POSITION;
	float2 Tex 				: TEXCOORD0;
	float2 Tex2 			: TEXCOORD1;
	float2 Tex3  			: TEXCOORD2;  	
	float4 WDepth			: TEXCOORD3;
	float4 P					: TEXCOORD4;	
	float3 Normal 		: NORMAL;			
};

struct VS_OUTPUT
{
  float4 Pos 				: POSITION;
  float4 Tex 				: TEXCOORD0;
	float2 Tex2 			: TEXCOORD1;
 	float2 Tex3   		: TEXCOORD2;
 	float4 WDepth			: TEXCOORD3;    
 	float4 P					: TEXCOORD4; 	
	float3 Normal 		: TEXCOORD5;
	float4 RefrProj   : TEXCOORD6;
 	float4 ReflProj		: TEXCOORD7; 	
 	float3 FresVec		: TEXCOORD8; 	
};

float PhongSpecular(float3 normal, float3 viewDir, float specularDecay)
{
    float nDotL = dot(normal, lightSourceDir);
    float3 reflection = 2.0f * normal * nDotL + -lightSourceDir;
    reflection = normalize(reflection);
    float rdotV = saturate(dot(reflection, viewDir));
    return pow(rdotV, specularDecay);
}

void VS (	 in VS_INPUT IN, out VS_OUTPUT OUT )
{
  // UVs /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
  float3 WPos = mul( IN.Pos, matWorld );
	float2 Uv = ( WPos.xz / WaterScale );
	float4 WaterUv;
				 WaterUv.xy = Uv + ( time * WaterSpeed.xy );
				 WaterUv.zw = Uv + ( time * WaterSpeed.zw );

	float2 WaterUv2;
				 WaterUv2.xy = Uv - ( time * ( WaterSpeed.xy * 10 ) );	       
	float2 WaterUv3;
				 WaterUv3.xy = Uv + ( time * ( WaterSpeed.xy * 10 ) );
	
	// Fresnel /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////  
	float3 ViewPos = ViewInv[3].xyz - WPos;	
	OUT.FresVec = mul( ViewPos, TBN );

	// Waves ///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	float4 ClipPos = mul( IN.Pos, matWorldViewProj );
	float3 Visi = abs( ClipPos.xzy ) < ( ClipPos.www * 2 + 1 );
	
	if (all(Visi)) {
		float HeightScale = 1 - pow ( saturate( dot( ViewPos.xz / ScaleVec.x, ViewPos.xz / ScaleVec.x ) ), 20 );
	  float Height = tex2Dlod( TexWaves, float4( WaterUv2.xy, 0, 0 ) * 0.5f ).w + tex2Dlod( TexWaves, float4( WaterUv2.xy, 0, 0 ) * 0.5f ).w;  
	  		  Height += tex2Dlod( TexWaves, float4( WaterUv3.xy, 0, 0 ) ).w + tex2Dlod( TexWaves2, float4( WaterUv3.xy, 0, 0 ) ).w;  
	  		  Height /= 1.5;	  
	  IN.Pos.y = ( IN.Pos.y + ( ( Height * WaveHeight ) - WaveHeight ) * HeightScale ) * ScaleVec.y;
	};
	
	OUT.Pos 					= mul( IN.Pos, matWorldViewProj );
	OUT.P				  		= mul( IN.Pos, matWorld );

	OUT.Tex 					= WaterUv;	
	OUT.Tex2					= WaterUv2;	
	OUT.Tex3					= WaterUv3;	

	OUT.WDepth  			= OUT.Pos;
	OUT.Normal				= normalize( mul( IN.Normal, matWorld ) );

	// ProjCoors ///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	OUT.RefrProj = float4( OUT.Pos.x * 0.5 + 0.5 * OUT.Pos.w, 0.5 * OUT.Pos.w - OUT.Pos.y * 0.5, OUT.Pos.w, OUT.Pos.w );
 	OUT.ReflProj = float4( OUT.Pos.x * 0.5 + 0.5 * OUT.Pos.w, 0.5 * OUT.Pos.w + OUT.Pos.y * 0.5, OUT.Pos.w, OUT.Pos.w );	

};

float4 PS (	in VS_OUTPUT IN ) : COLOR 
{

	float2 ProjCoords		= ( IN.ReflProj.xy / IN.RefrProj.z );
	float2 ProjCoords2	= ( IN.RefrProj.xy / IN.RefrProj.z );	

	// Soft edges //////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	float2 texelSize	= float2(1.0 / ResX, 1.0 / ResY );
	float2 offset     = texelSize / 2.0;		
	float SceneDepth	= tex2D(DEPTH, ProjCoords2 + offset );
	float Dist 				= distance(IN.P,CamPos);				
	float CurDepth		= IN.WDepth.z / IN.WDepth.w;	
	float DepthDiff		= ( SceneDepth - CurDepth ) * Dist * ( Softness * Dist );
	DepthDiff					= saturate( DepthDiff );

	float WaterAlpha 	= ( (DepthDiff * 7) * Alpha );
	

	// Normal //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	NormalWater				= tex2D( TexWaves, IN.Tex2 *.5 ) * 0.30f;
	NormalWater				+= tex2D( TexWaves, IN.Tex3 ) * 0.30f;	

	NormalWater			 	+= tex2D( TexNormalW, IN.Tex / ( TexScale ) + ( time * WaterSpeed.zw ) ) * 0.20f;
	NormalWater			 	+= tex2D( TexNormalN, IN.Tex / ( TexScale ) - ( time * WaterSpeed.zw ) ) * 0.20f;

	NormalWater				= normalize( NormalWater * 2.0f - 1.0f );


	// Reflection //////////////////////////////////////////////////////////////////////////////////////////////////////////////////
  float4 Refle 	= tex2D(TexReflection,((ProjCoords) + NormalWater.xy * Reflect ));
/*
        for(int i=1; i<13; i++) {
            Refle+=tex2D(TexReflection, (ProjCoords + NormalWater.xy * Reflect ) + pixelKernelV[i] ) * blurWeights[i]; 
            Refle+=tex2D(TexReflection, (ProjCoords + NormalWater.xy * Reflect ) + pixelKernel[i] ) * blurWeights[i]; 
        } 
   
	Refle /= 5;
*/   
  Refle 				*= float4( ReflectionColor , 1.0f );
  
	// Refraction with Achromatic aberration ///////////////////////////////////////////////////////////////////////////////////////
	float2 refrTC = (NormalWater.xy *(0.1+ 0.1)) * Refract;
	float4 Refra	= 0;
  Refra.x 			= tex2D(TexScreen,  ProjCoords2 + refrTC * 1.15 ).x;
  Refra.y 			= tex2D(TexScreen,  ProjCoords2 + refrTC ).y;
  Refra.z 			= tex2D(TexScreen,  ProjCoords2 + refrTC * 0.85 ).z;
	Refra 				= (( 1.0 - DepthDiff ) + float4( RefractionColor, 1 ) ) * Refra; // Color based on water depth
	
	// Fresnel /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	float3 Fresnel 	= dot( NormalWater, normalize( IN.FresVec ) );
	Fresnel 				= lerp( 0, float3(1,.8,.8), Fresnel );

	// Specular ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	float3 PosCamera		= normalize( CamPos - IN.P );
	float3 normalWorld 	= normalize((float3(0, 1, 0) * NormalWater.z) + (NormalWater.x * float3(0, 0, 1) + NormalWater.y * float3(-1, 0, 0)));
	float3 Specular 		= PhongSpecular( normalWorld, -PosCamera, 256 ) * float3(1.3, 1.2, 1.0); // Yellow tint
	//Specular += PhongSpecular(normalWorld, PosCamera, 1024) * float3(0.7, 0.7, 0.8);
	
 	// Foam ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////	
	float4 F1 	= tex2D( FoamT2, IN.Tex * 25 ) - DepthDiff * 4;
	float4 F2 	= tex2D( FoamT, (IN.Tex * 3) + ( NormalWater.xy / 100 ) ) - DepthDiff;
	float4 Foam = ( saturate( F1 ) + saturate( F2 ) ) / 2;	
	Foam 				= ambientColor * Foam + (Foam * saturate(dot(IN.Normal, -lightSourceDir)) * lightColor);	
	
	// Hori Fog ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
 	float Fog = saturate((Dist - 2000) / (5000 - 2000));

	// Final product ( Refle + Refla + Fresnel + Specular ) ////////////////////////////////////////////////////////////////////////
	float3 Water 	= lerp( Refle, Refra, Fresnel );
	Water.rgb 		+= Specular;
	Water.rgb 		+= Foam;	
 	Water.rgb 		= lerp( Water, FogColor, Fog );

	return float4(Water, WaterAlpha);
};


technique Water { 
	pass p {  
		AlphaBlendEnable	= true;
		SrcBlend					= SrcAlpha;
		DestBlend					= InvSrcAlpha;		
    
		VertexShader 	= compile vs_3_0 VS();
		PixelShader 	= compile ps_3_0 PS();
	}
}